/* TGE: subTree/subGraph/connected components Enumeration algorithm
            12/Feb/2002   by Takeaki Uno  e-mail:uno@nii.jp, 
    homepage:   http://research.nii.ac.jp/~uno/index.html  */

/* This program is available for only academic use, basically.
   Anyone can modify this program, but he/she has to write down 
    the change of the modification on the top of the source code.
   Neither contact nor appointment to Takeaki Uno is needed.
   If one wants to re-distribute this code, do not forget to 
    refer the newest code, and show the link to homepage of 
    Takeaki Uno, to notify the news about the codes for the users.
   For the commercial use, please make a contact to Takeaki Uno. */


#ifndef _TGE_c_
#define _TGE_c_

#include"agraph.c"
#include"problem.c"

#define TGE_ROOT 1
#define TGE_TREE 2
#define TGE_GRAPH 4
#define TGE_COMPONENT 8

void TGE_error (){
  ERROR_MES = "command explanation";
  print_err ("TGE: tTcCgGdDqV [options] input-filename [root vertex] [outputfile]\n\
enumerate all (spanning, directed, rooted,) trees/forests\n\
%%:show progress, _:no message, +:write solutions in append mode\n\
t:subtrees, T:trees rooted at [root vertex]\n\
c:connected components, C:connected components including [root vertex]\n\
g:connected subgraphs, G:connected subgraphs including [root vertex]\n\
d:directed graph, D:directed graph of reverse direction\n\
[options]\n\
-l [num]:output trees/graphs with at least [num] vertices\n\
-u [num]:output trees/graphs with at most [num] vertices\n\
-# [num]:stop after outputting [num] solutions\n\
-, [char]:give the separator of the numbers in the output\n\
-Q [filename]:replace the output numbers according to the permutation table given by [filename]\n\
# the 1st letter of input-filename cannot be '-'.\n\
# if the output file name is -, the solutions will be output to standard output.\n");
  EXIT;
}


/***********************************************************************/
/*  read parameters given by command line  */
/***********************************************************************/
void TGE_read_param (int argc, char *argv[], PROBLEM *PP){
  int c=1;
  ITEMSET *II = &PP->II;
  if ( argc < c+2 ){ TGE_error (); return; }
  
  if ( !strchr (argv[c], '_') ){ II->flag |= SHOW_MESSAGE; PP->AG.flag |= SHOW_MESSAGE; }
  if ( strchr (argv[c], '%') ) II->flag |= SHOW_PROGRESS;
  if ( strchr (argv[c], '+') ) II->flag |= ITEMSET_APPEND;
  if ( strchr (argv[c], 't') ){ PP->problem = TGE_TREE; II->flag |= ITEMSET_OUTPUT_EDGE; }
  else if ( strchr (argv[c], 'T') ){ PP->problem = TGE_TREE+TGE_ROOT; II->flag |= ITEMSET_OUTPUT_EDGE;}
  else if ( strchr (argv[c], 'c') ) PP->problem = TGE_COMPONENT;
  else if ( strchr (argv[c], 'C') ) PP->problem = TGE_COMPONENT + TGE_ROOT;
  else if ( strchr (argv[c], 'g') ){ PP->problem = TGE_GRAPH; II->flag |= ITEMSET_OUTPUT_EDGE; }
  else if ( strchr (argv[c], 'G') ){ PP->problem = TGE_GRAPH + TGE_ROOT; II->flag |= ITEMSET_OUTPUT_EDGE; }
  else error ("t, T, c, C, g or G command has to be specified", EXIT);

  if ( strchr (argv[c], 'd') ) II->dir = 1;
  if ( strchr (argv[c], 'D') ) II->dir = -1;
  c++;
  

  while ( argv[c][0] == '-' ){
    switch (argv[c][1]){
//    if ( strchr( argv[c], 'K' ) ) II->topk.end = atoi (argv[c+1]);
      case 'l': II->lb = atoi (argv[c+1]);
      break; case 'u': II->ub = atoi(argv[c+1]);
//    if ( strchr( argv[c], 'U' ) ) II->frq_ub = (WEIGHT)atof(argv[c+1]);
//    if ( strchr( argv[c], 'w' ) ) PP->weight_fname = argv[c+1];
      break; case '#': II->max_solutions = atoi(argv[c+1]);
      break; case ',': II->separator = argv[c+1][0];
      break; case 'Q': PP->outperm_fname = argv[c+1];
      break; default: goto NEXT;
    }
    c += 2;
    if ( argc < c+1 ){ TGE_error (); return; }
  }

  NEXT:;
  PP->AG.fname = argv[c];
  if ( PP->problem & TGE_ROOT ){
    if ( argc<=c+1 ) error ("root vertex is not given", EXIT);
    PP->root = atoi(argv[c+1]);
    c++;
  }
//  if ( II->topk.end==0 ) II->frq_lb = (WEIGHT)atof(argv[c+1]);
  if ( argc>c+1 ) PP->output_fname = argv[c+1];
}


    /**********************************************************************/
    /*   iteration of enumerting trees/subgraphs/components including r   */
    /*   most is the number of addible edges                              */
    /**********************************************************************/
void TGE_rooted_tree_iter (PROBLEM *PP, ALIST_ID r, ALIST_ID most){
  AGRAPH *G = &PP->AG;
  ITEMSET *II = &PP->II;
  QUEUE_ID s = PP->veccand.s;
  ALIST_ID e, vv, most2 = most;
  MALIST *L = AGRAPH_EDGES(*G, II->dir);
  II->iters++;

  while (1){
    e = MALIST_HEAD(*L,r);  /* choose one edge incident to r */
    if ( MALIST_IS_ELM(*L,e) ) break;
       /* if no edge incident to r, move to another vertex */
    if ( QUEUE_LENGTH_(PP->veccand)==0 ) goto END; /* if there is no more vertex to visit, end */
    r = QUEUE_ext_ (&PP->veccand);
  }
  vv = AGRAPH_INC_TO (*G, e, II->dir);
  if ( II->dir ) AGRAPH_arc_out (G, e, 0);
  else AGRAPH_edge_out (G, e, 0);  /* remove e from G */
  if ( (PP->problem&TGE_COMPONENT) && PP->vecflag[vv] == 0 ){
    AGRAPH_node_out (G, vv, 0);
    most--;
  } else if ( PP->problem & TGE_GRAPH ) most--;
  else if (( PP->problem & TGE_TREE ) && !MALIST_IS_ELM(*L,MALIST_HEAD(*L,vv)) ) most--;
  if ( II->itemset.t + most >= II->lb )
      TGE_rooted_tree_iter (PP, r, most);   /* recursive call for those not including e */
  if ( (PP->problem&TGE_COMPONENT) && PP->vecflag[vv] == 0 ) AGRAPH_node_recov (G, vv, 0);

  QUE_INS (II->itemset, e); /* add e to the current solution */
     /* if one endpoint of e does not belong to the current solution, 
       generate recursive call for those including e */
  if ( PP->vecflag[vv] == 0 ){
    ITEMSET_output_itemset (II, NULL, 0);  /* output solution */
    PP->vecflag[vv] = 1;   /* set the mark to the original */
    QUE_INS (PP->veccand, vv);
      /* if not exceed the upper bound, recursive call */
    if ( II->itemset.t < II->ub ) TGE_rooted_tree_iter (PP, r, most2);
    PP->vecflag[vv] = 0;   /* reset mark to the original */
    QUEUE_ext_tail_ (&PP->veccand);
  } else if ( PP->problem & TGE_GRAPH ){
    ITEMSET_output_itemset (II, NULL, 0);  /* output solution */
      /* if not exceed the upper bound, recursive call */
    if ( II->itemset.t < II->ub ) TGE_rooted_tree_iter (PP, r, most2);
  }
  QUEUE_ext_tail_ (&II->itemset);

  if ( II->dir ) AGRAPH_arc_recov (G, e, 0);
  else AGRAPH_edge_recov (G, e, 0);  /* recover e */

END:;
  PP->veccand.s = s;
}


 /*********************************************************************/
 /***    Enumerate rooted trees/subgraphs/connected components of G  **/
 /*********************************************************************/
void TGE_rooted_tree (PROBLEM *PP, ALIST_ID r){
  AGRAPH *G = &PP->AG;
  ALIST_ID most;
  PP->vecflag[r] = 1;
  PP->root = r;
  if ( PP->problem & TGE_GRAPH ) most = G->edge_num + G->arc_num;
  else most = G->node_num;
  TGE_rooted_tree_iter (PP, r, most);
  PP->vecflag[r] = 0;
}

 /*********************************************************************/
 /***    Enumerate subtree/subgraph/connected components of G       ***/
 /*********************************************************************/
void TGE_subtree (PROBLEM *PP){
  AGRAPH *G = &PP->AG;
  ITEMSET *II = &PP->II;
  ALIST_ID u;
  FLOOP (u, 0, G->node_end){
    TGE_rooted_tree (PP, u);
    if ( II->dir == 0 ) AGRAPH_node_out (G, u, 0);
  }
}


    /*****************************/
    /***   main routine        ***/
    /*****************************/
int TGE_main (int argc, char *argv[]){
  PROBLEM PP;
  
  PROBLEM_init (&PP);
  TGE_read_param (argc, argv, &PP);    if ( ERROR_MES ) return (1);

  PP.AG.flag |= PP.II.dir?LOAD_ARC:LOAD_EDGE;
  PROBLEM_load (&PP);      if ( ERROR_MES ) return(1);
  PROBLEM_alloc (&PP, MAX(PP.AG.edge_num,PP.AG.arc_num), PP.AG.node_num, MAX(PP.AG.edge_num,PP.AG.arc_num), NULL, PROBLEM_VECFLAG + PROBLEM_VECCAND);
  PP.II.X = &PP.AG;
  
  if ( (PP.problem & TGE_ROOT) && !RANGE(0, PP.root, PP.AG.node_end-1) )
      error_num ("root vertex is out of range, %d", PP.root, return (1));
  
  if ( !ERROR_MES ){
    if ( PP.problem & TGE_ROOT ) TGE_rooted_tree (&PP, PP.root);
    else TGE_subtree (&PP);
    ITEMSET_last_output (&PP.II);
  }
  
  PROBLEM_end (&PP);
  return (ERROR_MES?1:0);
}

/*******************************************************************************/
#ifndef _NO_MAIN_
#define _NO_MAIN_
int main (int argc, char *argv[]){
  return (TGE_main (argc, argv));
}
#endif
/*******************************************************************************/

#endif
